MODULE asmphyto3dbal_medusa
   !!======================================================================
   !!                       ***  MODULE asmphyto2dbal_medusa  ***
   !! Calculate increments to MEDUSA based on surface phyto2d increments
   !!
   !! IMPORTANT NOTE: This calls the bioanalysis routine of Hemmings et al.
   !! For licensing reasons this is kept in its own internal Met Office
   !! branch (dev/frdf/vn3.6_nitrogen_balancing) rather than in the Paris
   !! repository, and must be merged in when building.
   !!
   !!======================================================================
   !! History :  3.6  ! 2017-08 (D. Ford)  Adapted from asmphyto2dbal_hadocc
   !!----------------------------------------------------------------------
#if defined key_asminc && defined key_medusa && defined key_foam_medusa
   !!----------------------------------------------------------------------
   !! 'key_asminc'          : assimilation increment interface
   !! 'key_medusa'          : MEDUSA model
   !! 'key_foam_medusa'     : MEDUSA extras for FOAM OBS and ASM
   !!----------------------------------------------------------------------
   !! asm_phyto2d_bal_medusa : routine to calculate increments to MEDUSA
   !!----------------------------------------------------------------------
   USE par_kind,      ONLY: wp             ! kind parameters
   USE par_oce,       ONLY: jpi, jpj, jpk  ! domain array sizes
   USE dom_oce,       ONLY: gdepw_n        ! domain information
   USE zdftmx,        ONLY: ln_tmx_itf, &  ! Indonesian Throughflow
      &                     mask_itf       ! tidal mixing mask
   USE iom                                 ! i/o
   USE sms_medusa                          ! MEDUSA parameters
   USE par_medusa                          ! MEDUSA parameters
   USE par_trc,       ONLY: jptra          ! Tracer parameters
   USE bioanalysis                         ! Nitrogen balancing

   IMPLICIT NONE
   PRIVATE                   

   PUBLIC asm_phyto3d_bal_medusa

   ! Default values for biological assimilation parameters
   ! Should match Hemmings et al. (2008)
   REAL(wp), PARAMETER :: balnutext  =  0.6    !: Default nutrient balancing factor
   REAL(wp), PARAMETER :: balnutmin  =  0.1    !: Fraction of phytoplankton loss to nutrient
   REAL(wp), PARAMETER :: r          =  1      !: Reliability of model specific growth rate
   REAL(wp), PARAMETER :: beta_g     =  0.05   !: Low rate bias correction for growth rate estimator
   REAL(wp), PARAMETER :: beta_l     =  0.05   !: Low rate bias correction for primary loss rate estimator
   REAL(wp), PARAMETER :: beta_m     =  0.05   !: Low rate bias correction for secondary loss rate estimator
   REAL(wp), PARAMETER :: a_g        =  0.2    !: Error s.d. for log10 of growth rate estimator
   REAL(wp), PARAMETER :: a_l        =  0.4    !: Error s.d. for log10 of primary loss rate estimator
   REAL(wp), PARAMETER :: a_m        =  0.7    !: Error s.d. for log10 of secondary loss rate estimator
   REAL(wp), PARAMETER :: zfracb0    =  0.7    !: Base zooplankton fraction of loss to Z & D
   REAL(wp), PARAMETER :: zfracb1    =  0      !: Phytoplankton sensitivity of zooplankton fraction
   REAL(wp), PARAMETER :: qrfmax     =  1.1    !: Maximum nutrient limitation reduction factor
   REAL(wp), PARAMETER :: qafmax     =  1.1    !: Maximum nutrient limitation amplification factor
   REAL(wp), PARAMETER :: zrfmax     =  2      !: Maximum zooplankton reduction factor
   REAL(wp), PARAMETER :: zafmax     =  2      !: Maximum zooplankton amplification factor
   REAL(wp), PARAMETER :: prfmax     =  10     !: Maximum phytoplankton reduction factor (secondary)
   REAL(wp), PARAMETER :: incphymin  =  0.0001 !: Minimum size of non-zero phytoplankton increment
   REAL(wp), PARAMETER :: integnstep =  20     !: Number of steps for p.d.f. integral evaluation
   REAL(wp), PARAMETER :: pthreshold =  0.01   !: Fractional threshold level for setting p.d.f.
   !
   LOGICAL,  PARAMETER :: diag_active           = .TRUE.  !: Depth-independent diagnostics
   LOGICAL,  PARAMETER :: diag_fulldepth_active = .TRUE.  !: Full-depth diagnostics
   LOGICAL,  PARAMETER :: gl_active             = .TRUE.  !: Growth/loss-based balancing
   LOGICAL,  PARAMETER :: nbal_active           = .TRUE.  !: Nitrogen balancing
   LOGICAL,  PARAMETER :: subsurf_active        = .TRUE.  !: Increments below MLD
   LOGICAL,  PARAMETER :: deepneg_active        = .FALSE. !: Negative primary increments below MLD
   LOGICAL,  PARAMETER :: deeppos_active        = .FALSE. !: Positive primary increments below MLD
   LOGICAL,  PARAMETER :: nutprof_active        = .TRUE.  !: Secondary increments

CONTAINS

   SUBROUTINE asm_phyto3d_bal_medusa( pinc_chltot,                    &
      &                               pincper,                        &
      &                               p_maxchlinc,                    &
      &                               pgrow_avg_bkg, ploss_avg_bkg,   &
      &                               phyt_avg_bkg,                   &
      &                               tracer_bkg, phyto3d_balinc )
      !!---------------------------------------------------------------------------
      !!                    ***  ROUTINE asm_phyto2d_bal_medusa  ***
      !!
      !! ** Purpose :   calculate increments to MEDUSA from 2d phytoplankton increments
      !!
      !! ** Method  :   average up MEDUSA to look like HadOCC
      !!                call nitrogen balancing scheme
      !!                separate back out to MEDUSA
      !!
      !! ** Action  :   populate phyto2d_balinc
      !!
      !! References :   Hemmings et al., 2008, J. Mar. Res.
      !!                Ford et al., 2012, Ocean Sci.
      !!---------------------------------------------------------------------------
      !!
      REAL(wp), INTENT(inout), DIMENSION(jpi,jpj,jpk)       :: pinc_chltot    ! chltot increments
      REAL(wp), INTENT(in   )                               :: pincper        ! Assimilation period
      REAL(wp), INTENT(in   )                               :: p_maxchlinc    ! Max chl increment
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,jpk)       :: pgrow_avg_bkg  ! Avg phyto growth
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,jpk)       :: ploss_avg_bkg  ! Avg phyto loss
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,jpk)       :: phyt_avg_bkg   ! Avg phyto
      REAL(wp), INTENT(in   ), DIMENSION(jpi,jpj,jpk,jptra) :: tracer_bkg     ! State variables
      REAL(wp), INTENT(  out), DIMENSION(jpi,jpj,jpk,jptra) :: phyto3d_balinc ! Balancing increments
      !!
      INTEGER                                               :: ji, jj, jk, jn ! Loop counters
      INTEGER                                               :: jkmax          ! Loop index
      INTEGER,                 DIMENSION(6)                 :: i_tracer       ! Tracer indices
      REAL(wp)                                              :: n2be_p         ! N:biomass for total phy
      REAL(wp)                                              :: n2be_z         ! N:biomass for total zoo
      REAL(wp)                                              :: n2be_d         ! N:biomass for detritus
      REAL(wp)                                              :: zfrac          ! Fraction
      REAL(wp)                                              :: zfrac_chn      ! Fraction of jpchn
      REAL(wp)                                              :: zfrac_chd      ! Fraction of jpchd
      REAL(wp)                                              :: zfrac_phn      ! Fraction of jpphn
      REAL(wp)                                              :: zfrac_phd      ! Fraction of jpphd
      REAL(wp)                                              :: zfrac_zmi      ! Fraction of jpzmi
      REAL(wp)                                              :: zfrac_zme      ! Fraction of jpzme
      REAL(wp)                                              :: zrat_pds_phd   ! Ratio of jppds:jpphd
      REAL(wp)                                              :: zrat_chd_phd   ! Ratio of jpchd:jpphd
      REAL(wp)                                              :: zrat_chn_phn   ! Ratio of jpchn:jpphn
      REAL(wp)                                              :: zrat_phn_chn   ! Ratio of jpphn:jpchn
      REAL(wp)                                              :: zrat_phd_chd   ! Ratio of jpphd:jpchd
      REAL(wp)                                              :: zrat_pds_chd   ! Ratio of jppds:jpchd
      REAL(wp)                                              :: zrat_dtc_det   ! Ratio of jpdtc:jpdet
      REAL(wp),                DIMENSION(jpi,jpj,jpk)       :: cchl_p         ! C:Chl for total phy
      REAL(wp),                DIMENSION(jpi,jpj)           :: cchl_p_2d      ! C:Chl for total phy
      REAL(wp),                DIMENSION(16)                :: modparm        ! Model parameters
      REAL(wp),                DIMENSION(20)                :: assimparm      ! Assimilation parameters
      REAL(wp),                DIMENSION(jpi,jpj,jpk,6)     :: bstate         ! Background state
      REAL(wp),                DIMENSION(jpi,jpj,1,6)       :: bstate_2d      ! Background state
      REAL(wp),                DIMENSION(jpi,jpj,jpk,6)     :: outincs        ! Balancing increments
      REAL(wp),                DIMENSION(jpi,jpj,1,6)       :: outincs_2d     ! Balancing increments
      REAL(wp),                DIMENSION(jpi,jpj,22)        :: diag           ! Depth-indep diagnostics
      REAL(wp),                DIMENSION(jpi,jpj,1,22)      :: diag_fulldepth ! Full-depth diagnostics
      REAL(wp),                DIMENSION(jpi,jpj)           :: pinc_chltot_2d
      REAL(wp),                DIMENSION(jpi,jpj)           :: pgrow_avg_bkg_2d
      REAL(wp),                DIMENSION(jpi,jpj)           :: ploss_avg_bkg_2d
      REAL(wp),                DIMENSION(jpi,jpj)           :: phyt_avg_bkg_2d
      REAL(wp),                DIMENSION(jpi,jpj,1)         :: tmask_2d
      REAL(wp),                DIMENSION(jpi,jpj)           :: pmld_2d
      REAL(wp),                DIMENSION(jpi,jpj)           :: mld_max_bkg_2d
      !!---------------------------------------------------------------------------

      ! If p_maxchlinc > 0 then cap total absolute chlorophyll increment at that value
      IF ( p_maxchlinc > 0.0 ) THEN
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  pinc_chltot(ji,jj,jk) = MAX( -1.0 * p_maxchlinc, MIN( pinc_chltot(ji,jj,jk), p_maxchlinc ) )
               END DO
            END DO
         END DO
      ENDIF

      ! Set up model parameters to be passed into Hemmings balancing routine.
      ! For now these are hardwired to the standard HadOCC parameter values
      ! (except C:N ratios) as this is what the scheme was developed for.
      ! Obviously, HadOCC and MEDUSA are rather different models, so this
      ! isn't ideal, but there's not always direct analogues between the two
      ! parameter sets, so it's the easiest way to get something running.
      ! In the longer term, some serious MarMOT-based development is required.
      modparm(1)  = 0.1                               ! grow_sat
      modparm(2)  = 2.0                               ! psmax
      modparm(3)  = 0.845                             ! par
      modparm(4)  = 0.02                              ! alpha
      modparm(5)  = 0.05                              ! resp_rate
      modparm(6)  = 0.05                              ! pmort_rate
      modparm(7)  = 0.01                              ! phyto_min
      modparm(8)  = 0.05                              ! z_mort_1
      modparm(9)  = 1.0                               ! z_mort_2
      modparm(10) = ( xthetapn  + xthetapd  ) / 2.0   ! c2n_p
      modparm(11) = ( xthetazmi + xthetazme ) / 2.0   ! c2n_z
      modparm(12) = xthetad                           ! c2n_d
      modparm(13) = 0.01                              ! graze_threshold
      modparm(14) = 2.0                               ! holling_coef
      modparm(15) = 0.5                               ! graze_sat
      modparm(16) = 2.0                               ! graze_max

      ! Set up assimilation parameters to be passed into balancing routine
      ! Not sure what assimparm(1) is meant to be, but it doesn't get used
      assimparm(2)  = balnutext
      assimparm(3)  = balnutmin
      assimparm(4)  = r
      assimparm(5)  = beta_g
      assimparm(6)  = beta_l
      assimparm(7)  = beta_m
      assimparm(8)  = a_g
      assimparm(9)  = a_l
      assimparm(10) = a_m
      assimparm(11) = zfracb0
      assimparm(12) = zfracb1
      assimparm(13) = qrfmax
      assimparm(14) = qafmax
      assimparm(15) = zrfmax
      assimparm(16) = zafmax
      assimparm(17) = prfmax
      assimparm(18) = incphymin
      assimparm(19) = integnstep
      assimparm(20) = pthreshold

      ! Set up external tracer indices array bstate
      i_tracer(1) = 1   ! nutrient
      i_tracer(2) = 2   ! phytoplankton
      i_tracer(3) = 3   ! zooplankton
      i_tracer(4) = 4   ! detritus
      i_tracer(5) = 5   ! DIC
      i_tracer(6) = 6   ! Alkalinity

      ! Set background state
      bstate(:,:,:,i_tracer(1)) = tracer_bkg(:,:,:,jpdin)
      bstate(:,:,:,i_tracer(2)) = tracer_bkg(:,:,:,jpphn) + tracer_bkg(:,:,:,jpphd)
      bstate(:,:,:,i_tracer(3)) = tracer_bkg(:,:,:,jpzmi) + tracer_bkg(:,:,:,jpzme)
      bstate(:,:,:,i_tracer(4)) = tracer_bkg(:,:,:,jpdet)
      bstate(:,:,:,i_tracer(5)) = tracer_bkg(:,:,:,jpdic)
      bstate(:,:,:,i_tracer(6)) = tracer_bkg(:,:,:,jpalk)

      ! Calculate carbon to chlorophyll ratio for combined phytoplankton
      ! and nitrogen to biomass equivalent for PZD
      ! Hardwire nitrogen mass to 14.01 for now as it doesn't seem to be set in MEDUSA
      cchl_p(:,:,:) = 0.0
      DO jk = 1, jpk
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF ( ( tracer_bkg(ji,jj,jk,jpchn) + tracer_bkg(ji,jj,jk,jpchd ) ) .GT. 0.0 ) THEN
                  cchl_p(ji,jj,jk) = xmassc * ( ( tracer_bkg(ji,jj,jk,jpphn) * xthetapn ) +      &
                     &                       ( tracer_bkg(ji,jj,jk,jpphd) * xthetapd )   ) /  &
                     &            ( tracer_bkg(ji,jj,jk,jpchn) + tracer_bkg(ji,jj,jk,jpchd ) )
               ENDIF
            END DO
         END DO
      END DO
      n2be_p = 14.01 + ( xmassc * ( ( xthetapn  + xthetapd  ) / 2.0 ) )
      n2be_z = 14.01 + ( xmassc * ( ( xthetazmi + xthetazme ) / 2.0 ) )
      n2be_d = 14.01 + ( xmassc * xthetad )

      pmld_2d(:,:)        = 0.5
      mld_max_bkg_2d(:,:) = 0.5

      DO jk = 1, jpk

         tmask_2d(:,:,1)       = tmask(:,:,jk)
         pinc_chltot_2d(:,:)   = pinc_chltot(:,:,jk)
         cchl_p_2d(:,:)        = cchl_p(:,:,jk)
         phyt_avg_bkg_2d(:,:)  = phyt_avg_bkg(:,:,jk)
         pgrow_avg_bkg_2d(:,:) = pgrow_avg_bkg(:,:,jk)
         ploss_avg_bkg_2d(:,:) = ploss_avg_bkg(:,:,jk)
         bstate_2d(:,:,1,:)    = bstate(:,:,jk,:)
         outincs_2d(:,:,:,:)   = 0.0
         
         ! Call nitrogen balancing routine
         CALL bio_analysis( jpi, jpj, 1, gdepw_n(:,:,2), i_tracer, modparm,   &
            &               n2be_p, n2be_z, n2be_d, assimparm,                      &
            &               INT(pincper), 1, INT(SUM(tmask_2d,3)), tmask_2d(:,:,:),       &
            &               pmld_2d(:,:), mld_max_bkg_2d(:,:), pinc_chltot_2d(:,:), cchl_p_2d(:,:), &
            &               nbal_active, phyt_avg_bkg_2d(:,:),                         &
            &               gl_active, pgrow_avg_bkg_2d(:,:), ploss_avg_bkg_2d(:,:),      &
            &               subsurf_active, deepneg_active,                         &
            &               deeppos_active, nutprof_active,                         &
            &               bstate_2d, outincs_2d,                                        &
            &               diag_active, diag,                                      &
            &               diag_fulldepth_active, diag_fulldepth )

         outincs(:,:,jk,:) = outincs_2d(:,:,1,:)

      END DO

      ! Loop over each grid point partioning the increments
      phyto3d_balinc(:,:,:,:) = 0.0
      DO jk = 1, jpk
         DO jj = 1, jpj
            DO ji = 1, jpi

               ! Phytoplankton
               IF ( ( tracer_bkg(ji,jj,jk,jpphn) > 0.0 ) .AND. &
                  & ( tracer_bkg(ji,jj,jk,jpphd) > 0.0 ) .AND. &
                  & ( pinc_chltot(ji,jj,jk) /= 0.0 ) ) THEN
                  ! Phytoplankton nitrogen split up based on existing ratios
                  zfrac_phn = tracer_bkg(ji,jj,jk,jpphn) / &
                     &        (tracer_bkg(ji,jj,jk,jpphn) + tracer_bkg(ji,jj,jk,jpphd))
                  zfrac_phd = tracer_bkg(ji,jj,jk,jpphd) / &
                     &        (tracer_bkg(ji,jj,jk,jpphn) + tracer_bkg(ji,jj,jk,jpphd))

                  ! Phytoplankton silicate split up based on existing ratios
                  zrat_pds_phd = tracer_bkg(ji,jj,jk,jppds) / tracer_bkg(ji,jj,jk,jpphd)

                  ! Chlorophyll split up based on existing ratios to phytoplankton nitrogen
                  ! Not using pinc_chltot directly as it's only 2D
                  ! This method should give same results at surface as splitting pinc_chltot would
                  zrat_chn_phn = tracer_bkg(ji,jj,jk,jpchn) / tracer_bkg(ji,jj,jk,jpphn)
                  zrat_chd_phd = tracer_bkg(ji,jj,jk,jpchd) / tracer_bkg(ji,jj,jk,jpphd)

                  phyto3d_balinc(ji,jj,jk,jpphn) = outincs(ji,jj,jk,i_tracer(2)) * zfrac_phn
                  phyto3d_balinc(ji,jj,jk,jpphd) = outincs(ji,jj,jk,i_tracer(2)) * zfrac_phd
                  phyto3d_balinc(ji,jj,jk,jppds) = phyto3d_balinc(ji,jj,jk,jpphd) * zrat_pds_phd
                  phyto3d_balinc(ji,jj,jk,jpchn) = phyto3d_balinc(ji,jj,jk,jpphn) * zrat_chn_phn
                  phyto3d_balinc(ji,jj,jk,jpchd) = phyto3d_balinc(ji,jj,jk,jpphd) * zrat_chd_phd
               ENDIF

               ! Zooplankton nitrogen split up based on existing ratios
               IF ( ( tracer_bkg(ji,jj,jk,jpzmi) > 0.0 ) .AND. ( tracer_bkg(ji,jj,jk,jpzme) > 0.0 ) ) THEN
                  zfrac_zmi = tracer_bkg(ji,jj,jk,jpzmi) / &
                     &        (tracer_bkg(ji,jj,jk,jpzmi) + tracer_bkg(ji,jj,jk,jpzme))
                  zfrac_zme = tracer_bkg(ji,jj,jk,jpzme) / &
                     &        (tracer_bkg(ji,jj,jk,jpzmi) + tracer_bkg(ji,jj,jk,jpzme))
                  phyto3d_balinc(ji,jj,jk,jpzmi) = outincs(ji,jj,jk,i_tracer(3)) * zfrac_zmi
                  phyto3d_balinc(ji,jj,jk,jpzme) = outincs(ji,jj,jk,i_tracer(3)) * zfrac_zme
               ENDIF

               ! Nitrogen nutrient straight from balancing scheme
               phyto3d_balinc(ji,jj,jk,jpdin) = outincs(ji,jj,jk,i_tracer(1))

               ! Nitrogen detritus straight from balancing scheme
               phyto3d_balinc(ji,jj,jk,jpdet) = outincs(ji,jj,jk,i_tracer(4))

               ! DIC straight from balancing scheme
               phyto3d_balinc(ji,jj,jk,jpdic) = outincs(ji,jj,jk,i_tracer(5))

               ! Alkalinity straight from balancing scheme
               phyto3d_balinc(ji,jj,jk,jpalk) = outincs(ji,jj,jk,i_tracer(6))

               ! Remove diatom silicate increment from nutrient silicate to conserve mass
               IF ( ( tracer_bkg(ji,jj,jk,jpsil) - phyto3d_balinc(ji,jj,jk,jppds) ) > 0.0 ) THEN
                  phyto3d_balinc(ji,jj,jk,jpsil) = phyto3d_balinc(ji,jj,jk,jppds) * (-1.0)
               ENDIF

               ! Carbon detritus based on existing ratios
               IF ( ( tracer_bkg(ji,jj,jk,jpdet) > 0.0 ) .AND. ( tracer_bkg(ji,jj,jk,jpdtc) > 0.0 ) ) THEN
                  zrat_dtc_det = tracer_bkg(ji,jj,jk,jpdtc) / tracer_bkg(ji,jj,jk,jpdet)
                  phyto3d_balinc(ji,jj,jk,jpdtc) = phyto3d_balinc(ji,jj,jk,jpdet) * zrat_dtc_det
               ENDIF

               ! Do nothing with iron or oxygen for the time being
               phyto3d_balinc(ji,jj,jk,jpfer) = 0.0
               phyto3d_balinc(ji,jj,jk,jpoxy) = 0.0

            END DO
         END DO
      END DO
      
      ! If performing extra tidal mixing in the Indonesian Throughflow,
      ! increments have been found to make the carbon cycle unstable
      ! Therefore, mask these out
      IF ( ln_tmx_itf ) THEN
         DO jn = 1, jptra
            DO jk = 1, jpk
               phyto3d_balinc(:,:,jk,jn) = phyto3d_balinc(:,:,jk,jn) * ( 1.0 - mask_itf(:,:) )
            END DO
         END DO
      ENDIF

   END SUBROUTINE asm_phyto3d_bal_medusa

#else
   !!----------------------------------------------------------------------
   !!   Default option : Empty routine
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE asm_phyto3d_bal_medusa(pinc_chltot,                    &
      &                              pincper,                        &
      &                              p_maxchlinc,                    &
      &                              pgrow_avg_bkg, ploss_avg_bkg,   &
      &                              phyt_avg_bkg,                   &
      &                              tracer_bkg, phyto3d_balinc )
      REAL    :: pinc_chltot(:,:,:)
      REAL    :: pincper
      REAL    :: p_maxchlinc
      REAL    :: pgrow_avg_bkg(:,:,:)
      REAL    :: ploss_avg_bkg(:,:,:)
      REAL    :: phyt_avg_bkg(:,:,:)
      REAL    :: tracer_bkg(:,:,:,:)
      REAL    :: phyto2d_balinc(:,:,:,:)
      WRITE(*,*) 'asm_phyto3d_bal_medusa: You should not have seen this print! error?'
   END SUBROUTINE asm_phyto3d_bal_medusa
#endif

   !!======================================================================
END MODULE asmphyto3dbal_medusa
